@@ -8,7 +8,6 @@
 */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 #include "g_control.h"
 #include "g_her_metr.h"
@@ -43,21 +42,19 @@
 #define OCCIDENTAL 0
 #define ORIENTAL 1
 
+/* Location of first Kana in the occidental glyph arrays.  (Kana, unlike
+   Kanji, are placed in the occidental array, at the very end.) */
+#define BEGINNING_OF_KANA 4195
+
 /* forward references */
-static double _label_width_stroke __P((const unsigned short *label));
-static void _draw_hershey_glyph __P((int num, double charsize, int type, bool oblique));
-static void _draw_hershey_penup_stroke __P((double dx, double dy, double charsize, bool oblique));
-static void _draw_hershey_string __P((const unsigned short *string));
-static void _draw_hershey_stroke __P((bool pendown, double deltax, double deltay));
-static void _draw_stroke __P((bool pendown, double deltax, double deltay));
-static bool _composite_char __P((unsigned char *composite, unsigned char *character, unsigned char *accent));
+static bool _composite_char ____P((unsigned char *composite, unsigned char *character, unsigned char *accent));
 
 /* _draw_hershey_stroke() draws a stroke, taking into account the
    transformation from Hershey units to user units, and also the current
    transformation matrix (as set by the user).  _draw_stroke is similar,
    but takes arguments in user units. */
 
-static void
+void
 #ifdef _HAVE_PROTOS
 _draw_hershey_stroke (bool pendown, double deltax, double deltay)
 #else
@@ -71,7 +68,7 @@ _draw_hershey_stroke (pendown, deltax, deltay)
 		HERSHEY_UNITS_TO_USER_UNITS (deltay));
 }
 
-static void
+void
 #ifdef _HAVE_PROTOS
 _draw_stroke (bool pendown, double deltax, double deltay)
 #else
@@ -111,7 +108,7 @@ _g_flabelwidth_hershey (s)
   /* convert string to a codestring, including annotations */
   codestring = _controlify (s);
 
-  label_width = _label_width_stroke (codestring);
+  label_width = _label_width_hershey (codestring);
   free (codestring);
   
   return label_width;
@@ -121,11 +118,11 @@ _g_flabelwidth_hershey (s)
    to the case when the current Plotter font is a Hershey font */
 double
 #ifdef _HAVE_PROTOS
-_g_falabel_hershey (int x_justify, int y_justify, const unsigned char *s)
+_g_falabel_hershey (const unsigned char *s, int x_justify, int y_justify)
 #else
-_g_falabel_hershey (x_justify, y_justify, s)
-     int x_justify, y_justify;
+_g_falabel_hershey (s, x_justify, y_justify)
      const unsigned char *s;
+     int x_justify, y_justify;
 #endif
 {
   unsigned short *codestring;
@@ -140,7 +137,7 @@ _g_falabel_hershey (x_justify, y_justify, s)
   codestring = _controlify (s);
 
   /* dimensions of the string, in user units */
-  label_width = _label_width_stroke (codestring);
+  label_width = _label_width_hershey (codestring);
   label_height = HERSHEY_UNITS_TO_USER_UNITS(HERSHEY_HEIGHT);
   
   x_justify_c = (char)x_justify;
@@ -186,12 +183,12 @@ _g_falabel_hershey (x_justify, y_justify, s)
       break;
     }
 
-  /* save five relevant drawing attributes, and restore them later */
+  /* save relevant drawing attributes, and restore them later */
   {
     char *old_line_mode, *old_cap_mode, *old_join_mode;
-    double old_line_width;
     int old_fill_level;
     double oldposx, oldposy;
+    bool old_dash_array_in_effect;
 
     old_line_mode = (char *)_plot_xmalloc (strlen (_plotter->drawstate->line_mode) + 1);
     old_cap_mode = (char *)_plot_xmalloc (strlen (_plotter->drawstate->cap_mode) + 1);
@@ -202,16 +199,15 @@ _g_falabel_hershey (x_justify, y_justify, s)
     strcpy (old_line_mode, _plotter->drawstate->line_mode);
     strcpy (old_cap_mode, _plotter->drawstate->cap_mode);
     strcpy (old_join_mode, _plotter->drawstate->join_mode);
-    old_line_width = _plotter->drawstate->line_width;
     old_fill_level = _plotter->drawstate->fill_level;
+    old_dash_array_in_effect = _plotter->drawstate->dash_array_in_effect;
     
     /* Our choices for rendering: solid lines, rounded capitals and joins,
        a line width equal to slightly more than 1 Hershey unit, and
        transparent filling. */
     _plotter->linemod ("solid");
-    _plotter->capmod ("round");		/* options: butt/round/projecting */
-    _plotter->joinmod ("round");		/* options: miter/round/bevel */
-    _plotter->flinewidth (HERSHEY_UNITS_TO_USER_UNITS (HERSHEY_STROKE_WIDTH));
+    _plotter->capmod ("round");	/* options: butt/round/projecting */
+    _plotter->joinmod ("round"); /* options: miter/round/bevel */
     _plotter->filltype (0);
     
     /* move to take horizontal and vertical justification into account;
@@ -219,17 +215,18 @@ _g_falabel_hershey (x_justify, y_justify, s)
     _draw_stroke (false, 
 		 x_offset * label_width, y_offset * label_height);
     
-    /* call stroker on the sequence of strokes obtained from each char */
+    /* call stroker on the sequence of strokes obtained from each char (the
+       stroker may manipulate the line width) */
     _draw_hershey_string (codestring);
     
-    /* Restore original values of the drawing attributes, free storage.
-       endpath() will be invoked in here automatically, flushing the
-       created polyline object comprising the stroked text. */
+    /* Restore original values of relevant drawing attributes, free
+       storage.  endpath() will be invoked in here automatically, flushing
+       the created polyline object comprising the stroked text. */
     _plotter->linemod (old_line_mode);
     _plotter->capmod (old_cap_mode);
     _plotter->joinmod (old_join_mode);
-    _plotter->flinewidth (old_line_width);
     _plotter->filltype (old_fill_level);
+    _plotter->drawstate->dash_array_in_effect = old_dash_array_in_effect;
     
     free (old_line_mode);
     free (old_cap_mode);
@@ -269,14 +266,14 @@ _g_falabel_hershey (x_justify, y_justify, s)
    [N.B. In Bob Beach's original UGS character stroke generator,
    the +1/6's here were +2/9 instead.  Better?] */
 
-/* _label_width_stroke() computes the width (total delta x) of a
+/* _label_width_hershey() computes the width (total delta x) of a
    controlified character string to be rendered in a vector font, in user
    units */
-static double
+double
 #ifdef _HAVE_PROTOS
-_label_width_stroke (const unsigned short *label) 
+_label_width_hershey (const unsigned short *label) 
 #else
-_label_width_stroke (label) 
+_label_width_hershey (label) 
      const unsigned short *label;
 #endif
 { 
@@ -297,7 +294,7 @@ _label_width_stroke (label)
 	/* glyph was spec'd via an escape, not as a char in a font */
 	{
 	  glyphnum = c & GLYPH_SPEC;
-	  glyph = (const unsigned char *)(_occidental_vector_glyphs[glyphnum]);
+	  glyph = (const unsigned char *)(_occidental_hershey_glyphs[glyphnum]);
 	  
 	  if (*glyph != '\0')	/* nonempty glyph */
 	    /* 1st two chars are bounds */
@@ -307,7 +304,7 @@ _label_width_stroke (label)
 	/* glyph was spec'd via an escape, not as a char in a font */
 	{
 	  glyphnum = c & GLYPH_SPEC;
-	  glyph = (const unsigned char *)_oriental_vector_glyphs[glyphnum];
+	  glyph = (const unsigned char *)_oriental_hershey_glyphs[glyphnum];
 	  
 	  if (*glyph != '\0')	/* nonempty glyph */
 	    /* 1st two chars are bounds */
@@ -398,7 +395,7 @@ _label_width_stroke (label)
 	  raw_fontnum = (c >> FONT_SHIFT) & ONE_BYTE;
 	  
 	  c &= ~FONT_SPEC;	/* extract character proper */
-	  glyphnum = (_vector_font_info[raw_fontnum].chars)[c];
+	  glyphnum = (_hershey_font_info[raw_fontnum].chars)[c];
 
 	  /* could be a pseudo glyph number, e.g. an indication that
 	     character is composite */
@@ -409,7 +406,7 @@ _label_width_stroke (label)
 	      /* if so, use 1st element of composite character */
 	      composite = (unsigned char)c;
 	      if (_composite_char (&composite, &character, &accent))
-		glyphnum = (_vector_font_info[raw_fontnum].chars)[character];
+		glyphnum = (_hershey_font_info[raw_fontnum].chars)[character];
 	      else
 		glyphnum = UNDE; /* hope this won't happen */
 	    }
@@ -419,7 +416,7 @@ _label_width_stroke (label)
 	  if (glyphnum & KS)
 	    glyphnum -= KS;
 
-	  glyph = (const unsigned char *)(_occidental_vector_glyphs[glyphnum]);
+	  glyph = (const unsigned char *)(_occidental_hershey_glyphs[glyphnum]);
 	  if (*glyph != '\0')	/* nonempty glyph */
 	    /* 1st two chars are bounds */
 	    width += charsize * ((int)glyph[1] - (int)glyph[0]);
@@ -435,7 +432,7 @@ _label_width_stroke (label)
    specified in Hershey units.  Size scaling and obliquing (true/false) are
    specified.  This is used for repositioning during rendering of
    composite (accented) characters. */
-static void
+void
 #ifdef _HAVE_PROTOS
 _draw_hershey_penup_stroke(double dx, double dy, double charsize, bool oblique)
 #else
@@ -453,10 +450,10 @@ _draw_hershey_penup_stroke(dx, dy, charsize, oblique)
 			charsize * dy);
 }
 
-/* _draw_hershey_glyph() draws a raw Hershey glyph, specified by index in
-   the occidental or oriental glyph arrays.  Size scaling and obliquing
-   (true/false) are specified. */
-static void
+/* _draw_hershey_glyph() invokes move() and cont() to draw a raw Hershey
+   glyph, specified by index in the occidental or oriental glyph arrays.
+   Size scaling and obliquing (true/false) are specified. */
+void
 #ifdef _HAVE_PROTOS
 _draw_hershey_glyph (int glyphnum, double charsize, int type, bool oblique)
 #else
@@ -479,10 +476,10 @@ _draw_hershey_glyph (glyphnum, charsize, type, oblique)
     {
     case OCCIDENTAL:
     default:
-      glyph = (const unsigned char *)(_occidental_vector_glyphs[glyphnum]);
+      glyph = (const unsigned char *)(_occidental_hershey_glyphs[glyphnum]);
       break;
     case ORIENTAL:
-      glyph = (const unsigned char *)(_oriental_vector_glyphs[glyphnum]); 
+      glyph = (const unsigned char *)(_oriental_hershey_glyphs[glyphnum]); 
       break;
     }
 
@@ -525,9 +522,10 @@ _draw_hershey_glyph (glyphnum, charsize, type, oblique)
     }
 }
 
-/* _draw_hershey_string() will stroke a string beginning at the present
-   location, which will be taken to be on the string's baseline */
-static void
+/* _draw_hershey_string() strokes a string beginning at present location,
+   which is taken to be on the string's baseline.  Besides invoking move()
+   and cont(), it invokes linewidth(). */
+void
 #ifdef _HAVE_PROTOS
 _draw_hershey_string (const unsigned short *string)
 #else
@@ -539,239 +537,304 @@ _draw_hershey_string (string)
   const unsigned short *ptr = string;
   double charsize = 1.0;
   double saved_charsize = 1.0;
-  double saved_position_x = (_plotter->drawstate->pos).x;
-  double saved_position_y = (_plotter->drawstate->pos).y;
-  
-  while ((c = (*ptr)) != '\0') 
+  double saved_position_x = _plotter->drawstate->pos.x;
+  double saved_position_y = _plotter->drawstate->pos.y;
+  double old_line_width;
+  int line_width_type = 0;	/* 0,1,2 = unset,occidental,oriental */
+
+  /* save line width (will restore at end) */
+  old_line_width = _plotter->drawstate->line_width;
+
+  while ((c = (*ptr++)) != '\0')
     {
+      /* Check for the four possibilities: (1) a Hershey glyph specified by
+	 glyph number, (2) an oriental Hershey glyph specified by glyph
+	 number, (3) a control code, and (4) an ordinary font character,
+	 which will be mapped to a Hershey glyph by one of the tables in
+	 g_fontdb.c. */
+
       if (c & RAW_HERSHEY_GLYPH)
-	_draw_hershey_glyph (c & GLYPH_SPEC, charsize, OCCIDENTAL, false);
+	{
+	  if (line_width_type != 1)
+	    {
+	      _plotter->flinewidth 
+		(HERSHEY_UNITS_TO_USER_UNITS (HERSHEY_STROKE_WIDTH));
+	      line_width_type = 1;
+	    }
+	  _draw_hershey_glyph (c & GLYPH_SPEC, charsize, OCCIDENTAL, false);
+	}
 
       else if (c & RAW_ORIENTAL_HERSHEY_GLYPH)
-	_draw_hershey_glyph (c & GLYPH_SPEC, charsize, ORIENTAL, false);
-
-      else	/* not a raw Hershey glyph; a control code or a genuine char */
 	{
-	  if (c & CONTROL_CODE)	
-	    switch (c & ~CONTROL_CODE) /* parse control codes */
-	      {
-	      case C_BEGIN_SUPERSCRIPT :
-		_draw_hershey_stroke (false, 
-				      SUPERSCRIPT_DX * charsize * HERSHEY_EM,
-				      SUPERSCRIPT_DY * charsize * HERSHEY_EM);
-		charsize *= SCRIPTSIZE;
-		break;
-		
-	      case C_END_SUPERSCRIPT:
-		charsize /= SCRIPTSIZE;
-		_draw_hershey_stroke (false, 
-				      - SUPERSCRIPT_DX * charsize * HERSHEY_EM,
-				      - SUPERSCRIPT_DY * charsize * HERSHEY_EM);
-		break;
-		
-	      case C_BEGIN_SUBSCRIPT:
-		_draw_hershey_stroke (false, 
-				      SUBSCRIPT_DX * charsize * HERSHEY_EM,
-				      SUBSCRIPT_DY * charsize * HERSHEY_EM);
-		charsize *= SCRIPTSIZE;
-		break;
-		
-	      case C_END_SUBSCRIPT:
-		charsize /= SCRIPTSIZE;
-		_draw_hershey_stroke (false, 
-				      - SUBSCRIPT_DX * charsize * HERSHEY_EM,
-				      - SUBSCRIPT_DY * charsize * HERSHEY_EM);
-		break;
-		
-	      case C_PUSH_LOCATION:
-		saved_charsize = charsize;
-		saved_position_x = _plotter->drawstate->pos.x;
-		saved_position_y = _plotter->drawstate->pos.y;
-		break;
-		
-	      case C_POP_LOCATION:
-		charsize = saved_charsize;
-		_plotter->fmove (saved_position_x, saved_position_y);
-		break;
-		
-	      case C_RIGHT_ONE_EM:
-		_draw_hershey_stroke (false, charsize * HERSHEY_EM, 0.0);
-		break;
-		
-	      case C_RIGHT_HALF_EM:
-		_draw_hershey_stroke (false, charsize * HERSHEY_EM / 2.0, 0.0);
-		break;
-		
-	      case C_RIGHT_QUARTER_EM:
-		_draw_hershey_stroke (false, charsize * HERSHEY_EM / 4.0, 0.0);
-		break;
-		
-	      case C_RIGHT_SIXTH_EM:
-		_draw_hershey_stroke (false, charsize * HERSHEY_EM / 6.0, 0.0);
-		break;
-		
-	      case C_RIGHT_EIGHTH_EM:
-		_draw_hershey_stroke (false, charsize * HERSHEY_EM / 8.0, 0.0);
-		break;
-		
-	      case C_LEFT_ONE_EM:
-		_draw_hershey_stroke (false, - charsize * HERSHEY_EM, 0.0);
-		break;
-		
-	      case C_LEFT_HALF_EM:
-		_draw_hershey_stroke (false, - charsize * HERSHEY_EM / 2.0, 0.0);
-		break;
-		
-	      case C_LEFT_QUARTER_EM:
-		_draw_hershey_stroke (false, - charsize * HERSHEY_EM / 4.0, 0.0);
-		break;
-		
-	      case C_LEFT_SIXTH_EM:
-		_draw_hershey_stroke (false, - charsize * HERSHEY_EM / 6.0, 0.0);
-		break;
-		
-	      case C_LEFT_EIGHTH_EM:
-		_draw_hershey_stroke (false, - charsize * HERSHEY_EM / 8.0, 0.0);
-		break;
-		
-		/* unrecognized control code */
-	      default:
-		break;
-	      }
-      
-	  else			/* yow, an actual character */
+	  if (line_width_type != 2)
 	    {
-	      int raw_fontnum;
-	      int glyphnum;		/* glyph in Hershey array */
-	      int char_glyphnum, accent_glyphnum; /* for composite chars */
-	      int char_width, accent_width; /* for composite chars */
-	      const unsigned char *char_glyph, *accent_glyph;
-	      unsigned char composite, character, accent;
-	      bool oblique, small_kana = false;
-	      
-	      /* compute index of font, in table in g_fontdb.c */
-	      raw_fontnum = (c >> FONT_SHIFT) & ONE_BYTE;
-	      /* shear font?  (for HersheySans-Oblique, etc.) */
-	      oblique = _vector_font_info[raw_fontnum].obliquing;
-	      
-	      c &= ~FONT_SPEC;	/* extract character proper */
-	      glyphnum = (_vector_font_info[raw_fontnum].chars)[c];
-
-	      if (glyphnum & KS)
-		{
-		  glyphnum -= KS;
-		  small_kana = true;
-		}
-
-	      switch (glyphnum)
-		{
-		  /* special case: this is a composite (accented)
-                     character; search table in g_fontdb.c for it */
-		case ACC0:
-		case ACC1:
-		case ACC2:
-		  composite = (unsigned char)c;
-		  if (_composite_char (&composite, &character, &accent))
-		    {
-		      char_glyphnum = 
-			(_vector_font_info[raw_fontnum].chars)[character];
-		      accent_glyphnum = 
-			(_vector_font_info[raw_fontnum].chars)[accent];
-		    }
-		  else
-		    {		/* hope this won't happen */
-		      char_glyphnum = UNDE; 
-		      accent_glyphnum = 0;
-		    }
-		  char_glyph = 
-		    (const unsigned char *)_occidental_vector_glyphs[char_glyphnum];
-		  accent_glyph = 
-		    (const unsigned char *)_occidental_vector_glyphs[accent_glyphnum];
-	  
-		  if (*char_glyph != '\0')	/* nonempty glyph */
-		    /* 1st two chars are bounds, in Hershey units */
-		    char_width = (int)char_glyph[1] - (int)char_glyph[0];
-		  else
-		    char_width = 0;
-
-		  if (*accent_glyph != '\0')	/* nonempty glyph */
-		    /* 1st two chars are bounds, in Hershey units */
-		    accent_width = (int)accent_glyph[1] - (int)accent_glyph[0];
-		  else
-		    accent_width = 0;
-
-		  /* draw the character */
-		  _draw_hershey_glyph (char_glyphnum, charsize, 
-				       OCCIDENTAL, oblique);
-		  /* back up */
-		  _draw_hershey_penup_stroke (-0.5 * (double)char_width
-					      -0.5 * (double)accent_width,
-					      0.0, charsize, oblique);
-
-		  /* repositioning for uppercase and uppercase italic */
-		  if (glyphnum == ACC1)
-		    _draw_hershey_penup_stroke (0.0, 
-						(double)(ACCENT_UP_SHIFT),
-						charsize, oblique);
-		  else if (glyphnum == ACC2)
-		    _draw_hershey_penup_stroke ((double)(ACCENT_RIGHT_SHIFT),
-						(double)(ACCENT_UP_SHIFT),
-						charsize, oblique);
-
-		  /* draw the accent */
-		  _draw_hershey_glyph (accent_glyphnum, charsize, 
-				       OCCIDENTAL, oblique);
-
-		  /* undo special repositioning if any */
-		  if (glyphnum == ACC1)
-		    _draw_hershey_penup_stroke (0.0, 
-						-(double)(ACCENT_UP_SHIFT),
-						charsize, oblique);
-		  else if (glyphnum == ACC2)
-		    _draw_hershey_penup_stroke (-(double)(ACCENT_RIGHT_SHIFT),
-						-(double)(ACCENT_UP_SHIFT),
-						charsize, oblique);
-
-		  /* move forward, to end composite char where we should */
-		  _draw_hershey_penup_stroke (0.5 * (double)char_width
-					      -0.5 * (double)accent_width,
-					      0.0, charsize, oblique);
-		  break;
-
-		  /* not a composite character; just an ordinary glyph,
-		     though it could be a small Kana */
-		default:
-		  if (small_kana)
-		    {
-		      int kana_width;
-		      const unsigned char *kana_glyph;
-		      double shift = 0.5 * (1.0 - (SMALL_KANA_SIZE));
-
-		      kana_glyph = 
-			(const unsigned char *)_occidental_vector_glyphs[glyphnum];
-		      kana_width = (int)kana_glyph[1] - (int)kana_glyph[0];
-
-		      /* draw small Kana, preceded and followed by a penup
-                         stroke in order to traverse the full width of an
-                         ordinary Kana */
-		      _draw_hershey_penup_stroke (shift * (double)kana_width,
-						  0.0, charsize, oblique);
-		      _draw_hershey_glyph (glyphnum, 
-					   (SMALL_KANA_SIZE) * charsize,
-					   OCCIDENTAL, oblique);
-		      _draw_hershey_penup_stroke (shift * (double)kana_width,
-						  0.0, charsize, oblique);
-		    }
-		  else
-		    _draw_hershey_glyph (glyphnum, charsize, 
-					 OCCIDENTAL, oblique);
-		  break;
-		}
+	      _plotter->flinewidth 
+		(HERSHEY_UNITS_TO_USER_UNITS (HERSHEY_ORIENTAL_STROKE_WIDTH));
+	      line_width_type = 2;
 	    }
+	  _draw_hershey_glyph (c & GLYPH_SPEC, charsize, ORIENTAL, false);
 	}
+
+      else if (c & CONTROL_CODE)	
+	switch (c & ~CONTROL_CODE) /* parse control codes */
+	  {
+	  case C_BEGIN_SUPERSCRIPT :
+	    _draw_hershey_stroke (false, 
+				  SUPERSCRIPT_DX * charsize * HERSHEY_EM,
+				  SUPERSCRIPT_DY * charsize * HERSHEY_EM);
+	    charsize *= SCRIPTSIZE;
+	    break;
+		
+	  case C_END_SUPERSCRIPT:
+	    charsize /= SCRIPTSIZE;
+	    _draw_hershey_stroke (false, 
+				  - SUPERSCRIPT_DX * charsize * HERSHEY_EM,
+				  - SUPERSCRIPT_DY * charsize * HERSHEY_EM);
+	    break;
+		
+	  case C_BEGIN_SUBSCRIPT:
+	    _draw_hershey_stroke (false, 
+				  SUBSCRIPT_DX * charsize * HERSHEY_EM,
+				  SUBSCRIPT_DY * charsize * HERSHEY_EM);
+	    charsize *= SCRIPTSIZE;
+	    break;
+		
+	  case C_END_SUBSCRIPT:
+	    charsize /= SCRIPTSIZE;
+	    _draw_hershey_stroke (false, 
+				  - SUBSCRIPT_DX * charsize * HERSHEY_EM,
+				  - SUBSCRIPT_DY * charsize * HERSHEY_EM);
+	    break;
+		
+	  case C_PUSH_LOCATION:
+	    saved_charsize = charsize;
+	    saved_position_x = _plotter->drawstate->pos.x;
+	    saved_position_y = _plotter->drawstate->pos.y;
+	    break;
+		
+	  case C_POP_LOCATION:
+	    charsize = saved_charsize;
+	    _plotter->fmove (saved_position_x, saved_position_y);
+	    break;
+		
+	  case C_RIGHT_ONE_EM:
+	    _draw_hershey_stroke (false, charsize * HERSHEY_EM, 0.0);
+	    break;
+		
+	  case C_RIGHT_HALF_EM:
+	    _draw_hershey_stroke (false, charsize * HERSHEY_EM / 2.0, 0.0);
+	    break;
+		
+	  case C_RIGHT_QUARTER_EM:
+	    _draw_hershey_stroke (false, charsize * HERSHEY_EM / 4.0, 0.0);
+	    break;
+		
+	  case C_RIGHT_SIXTH_EM:
+	    _draw_hershey_stroke (false, charsize * HERSHEY_EM / 6.0, 0.0);
+	    break;
+		
+	  case C_RIGHT_EIGHTH_EM:
+	    _draw_hershey_stroke (false, charsize * HERSHEY_EM / 8.0, 0.0);
+	    break;
+		
+	  case C_LEFT_ONE_EM:
+	    _draw_hershey_stroke (false, - charsize * HERSHEY_EM, 0.0);
+	    break;
+		
+	  case C_LEFT_HALF_EM:
+	    _draw_hershey_stroke (false, - charsize * HERSHEY_EM / 2.0, 0.0);
+	    break;
+		
+	  case C_LEFT_QUARTER_EM:
+	    _draw_hershey_stroke (false, - charsize * HERSHEY_EM / 4.0, 0.0);
+	    break;
+		
+	  case C_LEFT_SIXTH_EM:
+	    _draw_hershey_stroke (false, - charsize * HERSHEY_EM / 6.0, 0.0);
+	    break;
+		
+	  case C_LEFT_EIGHTH_EM:
+	    _draw_hershey_stroke (false, - charsize * HERSHEY_EM / 8.0, 0.0);
+	    break;
+		
+	    /* unrecognized control code, punt */
+	  default:
+	    break;
+	  }
       
-      ptr++;			/* bump pointer in controlified string */
-    }
+      else
+	/* yow, an actual font character!  Several possibilities: could be
+	   a composite (accented) character, could be a small Kana, or
+	   could be a garden-variety character. */
+	{
+	  int raw_fontnum;
+	  int glyphnum;		/* glyph in Hershey array */
+	  int char_glyphnum, accent_glyphnum; /* for composite chars */
+	  int char_width, accent_width; /* for composite chars */
+	  const unsigned char *char_glyph, *accent_glyph;
+	  unsigned char composite, character, accent;
+	  bool oblique, small_kana = false;
+	  
+	  /* compute index of font, in font table in g_fontdb.c */
+	  raw_fontnum = (c >> FONT_SHIFT) & ONE_BYTE;
+	  /* shear font?  (for HersheySans-Oblique, etc.) */
+	  oblique = _hershey_font_info[raw_fontnum].obliquing;
+	  
+	  c &= ~FONT_SPEC;	/* extract character proper */
+	  glyphnum = (_hershey_font_info[raw_fontnum].chars)[c];
+	  
+	  if (glyphnum & KS) /* a small kana? */
+	    {
+	      glyphnum -= KS;
+	      small_kana = true;
+	    }
+	  
+	  switch (glyphnum)
+	    {
+	      /* special case: this is a composite (accented) character;
+		 search font table in g_fontdb.c for it */
+	    case ACC0:
+	    case ACC1:
+	    case ACC2:
+	      composite = (unsigned char)c;
+	      if (_composite_char (&composite, &character, &accent))
+		{
+		  char_glyphnum = 
+		    (_hershey_font_info[raw_fontnum].chars)[character];
+		  accent_glyphnum = 
+		    (_hershey_font_info[raw_fontnum].chars)[accent];
+		}
+	      else
+		{		/* hope this won't happen */
+		  char_glyphnum = UNDE; 
+		  accent_glyphnum = 0;
+		}
+	      char_glyph = 
+		(const unsigned char *)_occidental_hershey_glyphs[char_glyphnum];
+	      accent_glyph = 
+		(const unsigned char *)_occidental_hershey_glyphs[accent_glyphnum];
+	  
+	      if (*char_glyph != '\0') /* nonempty glyph */
+		/* 1st two chars are bounds, in Hershey units */
+		char_width = (int)char_glyph[1] - (int)char_glyph[0];
+	      else
+		char_width = 0;
+
+	      if (*accent_glyph != '\0') /* nonempty glyph */
+		/* 1st two chars are bounds, in Hershey units */
+		accent_width = (int)accent_glyph[1] - (int)accent_glyph[0];
+	      else
+		accent_width = 0;
+
+	      /* draw the character */
+	      if (line_width_type != 1)
+		{
+		  _plotter->flinewidth 
+		    (HERSHEY_UNITS_TO_USER_UNITS (HERSHEY_STROKE_WIDTH));
+		  line_width_type = 1;
+		}
+	      _draw_hershey_glyph (char_glyphnum, charsize, 
+				   OCCIDENTAL, oblique);
+	      /* back up to draw accent */
+	      _draw_hershey_penup_stroke (-0.5 * (double)char_width
+					  -0.5 * (double)accent_width,
+					  0.0, charsize, oblique);
+
+	      /* repositioning for uppercase and uppercase italic */
+	      if (glyphnum == ACC1)
+		_draw_hershey_penup_stroke (0.0, 
+					    (double)(ACCENT_UP_SHIFT),
+					    charsize, oblique);
+	      else if (glyphnum == ACC2)
+		_draw_hershey_penup_stroke ((double)(ACCENT_RIGHT_SHIFT),
+					    (double)(ACCENT_UP_SHIFT),
+					    charsize, oblique);
+
+	      /* draw the accent */
+	      _draw_hershey_glyph (accent_glyphnum, charsize, 
+				   OCCIDENTAL, oblique);
+
+	      /* undo special repositioning if any */
+	      if (glyphnum == ACC1)
+		_draw_hershey_penup_stroke (0.0, 
+					    -(double)(ACCENT_UP_SHIFT),
+					    charsize, oblique);
+	      else if (glyphnum == ACC2)
+		_draw_hershey_penup_stroke (-(double)(ACCENT_RIGHT_SHIFT),
+					    -(double)(ACCENT_UP_SHIFT),
+					    charsize, oblique);
+
+	      /* move forward, to end composite char where we should */
+	      _draw_hershey_penup_stroke (0.5 * (double)char_width
+					  -0.5 * (double)accent_width,
+					  0.0, charsize, oblique);
+	      break;
+
+	      /* not a composite (accented) character; just an ordinary
+		 glyph from occidental+Kana array (could be a Kana, in
+		 particular, could be a small Kana) */
+	    default:
+	      if (small_kana)
+		{
+		  int kana_width;
+		  const unsigned char *kana_glyph;
+		  double shift = 0.5 * (1.0 - (SMALL_KANA_SIZE));
+
+		  kana_glyph = 
+		    (const unsigned char *)_occidental_hershey_glyphs[glyphnum];
+		  kana_width = (int)kana_glyph[1] - (int)kana_glyph[0];
+
+		  /* draw small Kana, preceded and followed by a penup
+		     stroke in order to traverse the full width of an
+		     ordinary Kana */
+		  _draw_hershey_penup_stroke (shift * (double)kana_width,
+					      0.0, charsize, oblique);
+		  if (line_width_type != 2)
+		    {
+		      _plotter->flinewidth 
+			(HERSHEY_UNITS_TO_USER_UNITS (HERSHEY_ORIENTAL_STROKE_WIDTH));
+		      line_width_type = 2;
+		    }
+		  _draw_hershey_glyph (glyphnum, 
+				       (SMALL_KANA_SIZE) * charsize,
+				       OCCIDENTAL, oblique);
+		  _draw_hershey_penup_stroke (shift * (double)kana_width,
+					      0.0, charsize, oblique);
+		}
+	      else
+		/* whew! just an ordinary glyph from the occidental array
+		   (could be a Kana however, since they're confusingly
+		   placed in that array, at the end) */
+		{
+		  if (glyphnum >= BEGINNING_OF_KANA)
+		    {
+		      if (line_width_type != 2)
+			{
+			  _plotter->flinewidth 
+			    (HERSHEY_UNITS_TO_USER_UNITS (HERSHEY_ORIENTAL_STROKE_WIDTH));
+			  line_width_type = 2;
+			}
+		    }
+		  else
+		      if (line_width_type != 1)
+			{
+			  _plotter->flinewidth 
+			    (HERSHEY_UNITS_TO_USER_UNITS (HERSHEY_STROKE_WIDTH));
+			  line_width_type = 1;
+			}
+		_draw_hershey_glyph (glyphnum, charsize, 
+				     OCCIDENTAL, oblique);
+		}
+	      break;
+	    } /* end of case statement that switches based on glyphnum */
+
+	} /* end of font character case */
+
+    } /* end of loop through unsigned shorts in the codestring */
+  
+  if (line_width_type != 0)
+    /* must restore old line width */
+    _plotter->flinewidth (old_line_width);
   
   return;
 }
@@ -787,7 +850,7 @@ _composite_char (composite, character, accent)
      unsigned char *character, *accent;
 #endif
 {
-  const struct accented_char_info_struct *compchar = _vector_accented_char_info;
+  const struct accented_char_info_struct *compchar = _hershey_accented_char_info;
   bool found = false;
   unsigned char given = *composite;
   
