@@ -5,7 +5,6 @@
    bits, i.e. 0x0000 through 0xffff, for each of red, green, and blue). */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -25,9 +24,9 @@ _g_bgcolor (red, green, blue)
   if ((red > 0xffff) || (green > 0xffff) || (blue > 0xffff))
     /* OOB switches to default */
     {
-      red = _plotter->default_drawstate->bgcolor.red;
-      green = _plotter->default_drawstate->bgcolor.green;
-      blue = _plotter->default_drawstate->bgcolor.blue;
+      red = _default_drawstate.bgcolor.red;
+      green = _default_drawstate.bgcolor.green;
+      blue = _default_drawstate.bgcolor.blue;
     }
 
   /* save our notion of background color */
