@@ -1,22 +1,17 @@
 /* This file contains the closepl method, which is a standard part of
    libplot.  It closes a Plotter object. */
 
-/* This version is used for Plotters that emit graphics only after all
-   pages of graphics have been drawn, and the Plotter is deleted.  Such
-   Plotters maintain a linked list of pages (graphics are only written to
-   the output stream when a Plotter is deleted, and the appropriate
-   `terminate' method is invoked).  So this version simply finalizes the
-   current page by invoking endpath() etc.; it doesn't write anything out. */
+/* This generic version does very little, since GenericPlotters don't
+   emit any graphics code. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
 #ifdef _HAVE_PROTOS
-_g_closepl2(void)
+_g_closepl (void)
 #else
-_g_closepl2()
+_g_closepl ()
 #endif
 {
   if (!_plotter->open)
