@@ -9,7 +9,6 @@
    comment below. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -69,19 +68,17 @@ _g_fconcat (m0, m1, m2, m3, m4, m5)
   _plotter->drawstate->transform.nonreflection 
     = ((_plotter->flipped_y ? -1 : 1) * det >= 0) ? true : false;
 
-  /* recompute line width in device coordinates, being careful (if Plotter
-     is a MetaPlotter) not to cause a bogus LINEWIDTH op code to be emitted */
-  if (_plotter->type == PL_META)
-    _g_flinewidth (_plotter->drawstate->line_width);
-  else
-    _plotter->flinewidth (_plotter->drawstate->line_width);
+  /* Recompute line width in device coordinates.  In libplot, this is a
+     forwarding function; see api.c.  That's because for a MetaPlotter, it
+     should be almost a no-op, to avoid emitting a bogus LINEWIDTH op code.  */
+  _recompute_device_line_width ();
 
-  /* Even though we update the device-frame line width, we don't invoke the
-     retrieve_font operation to update the device-frame font size.  That is
-     because on X Plotters, it's not wise: the new device-frame font size
-     may be so small or large as to be unavailable on the X server, and the
-     user may in fact be planning to invoke fontsize() manually to select a
-     font of an appropriate size.
+  /* Even though we update the device-frame line width, we don't call the
+     retrieve_font function to update the device-frame font size.  That's
+     because on X Plotters, it wouldn't be wise: the new device-frame font
+     size may be so small or large as to be unavailable on the X server,
+     and the user may in fact be planning to invoke fontsize() manually to
+     select a font of an appropriate size.
 
      Even if the user doesn't plan on doing that, it's OK not to invoke
      retrieve_font here, since it'll be invoked before rendering any string
@@ -89,3 +86,14 @@ _g_fconcat (m0, m1, m2, m3, m4, m5)
 
   return 0;
 }
+
+/* generic (non-MetaPlotter) version of abovementioned function */
+void
+#ifdef _HAVE_PROTOS
+_g_recompute_device_line_width (void)
+#else
+_g_recompute_device_line_width ()
+#endif
+{
+  _plotter->flinewidth (_plotter->drawstate->line_width);
+}
