@@ -1,9 +1,8 @@
 /* This file contains the joinmod method, which is a GNU extension to
-   libplot.  It sets a drawing attribute: the join mode used when drawing
-   subsequent polylines. */
+   libplot.  It sets a drawing attribute: the join mode used when
+   subsequently drawing paths consisting of more than a single segment. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -25,7 +24,7 @@ _g_joinmod (s)
 
   /* null pointer resets to default */
   if ((!s) || !strcmp(s, "(null)"))
-    s = _plotter->default_drawstate->join_mode;
+    s = _default_drawstate.join_mode;
 
   free (_plotter->drawstate->join_mode);
   _plotter->drawstate->join_mode = (char *)_plot_xmalloc (strlen (s) + 1);
@@ -41,9 +40,11 @@ _g_joinmod (s)
     _plotter->drawstate->join_type = JOIN_ROUND;
   else if (strcmp( s, "bevel") == 0)
     _plotter->drawstate->join_type = JOIN_BEVEL;
+  else if (strcmp( s, "triangular") == 0)
+    _plotter->drawstate->join_type = JOIN_TRIANGULAR;
   else
     /* don't recognize, silently switch to default mode */
-    return _g_joinmod (_plotter->default_drawstate->join_mode);
+    return _g_joinmod (_default_drawstate.join_mode);
   
   return 0;
 }
