@@ -8,7 +8,6 @@
    bits, i.e. 0x0000 through 0xffff, for each of red, green, and blue). */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -31,9 +30,9 @@ _g_pencolor (red, green, blue)
   if ((red > 0xffff) || (green > 0xffff) || (blue > 0xffff))
     /* OOB switches to default */
     {
-      red = _plotter->default_drawstate->fgcolor.red;
-      green = _plotter->default_drawstate->fgcolor.green;
-      blue = _plotter->default_drawstate->fgcolor.blue;
+      red = _default_drawstate.fgcolor.red;
+      green = _default_drawstate.fgcolor.green;
+      blue = _default_drawstate.fgcolor.blue;
     }
 
   /* save our notion of foreground color */
