@@ -3,7 +3,6 @@
    x1,y1. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -20,20 +19,12 @@ _m_box (x0, y0, x1, y1)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d %d %d\n", 
-		 (int)O_BOX, x0, y0, x1, y1);
-      else
-	{
-	  putc ((int)O_BOX, _plotter->outstream);
-	  _emit_integer (x0);
-	  _emit_integer (y0);
-	  _emit_integer (x1);
-	  _emit_integer (y1);
-	}
-    }
+  _meta_emit_byte ((int)O_BOX);
+  _meta_emit_integer (x0);
+  _meta_emit_integer (y0);
+  _meta_emit_integer (x1);
+  _meta_emit_integer (y1);
+  _meta_emit_terminator ();
 
   return 0;
 }
@@ -52,20 +43,12 @@ _m_fbox (x0, y0, x1, y1)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %g %g %g %g\n", 
-		 (int)O_BOX, x0, y0, x1, y1);
-      else
-	{
-	  putc ((int)O_FBOX, _plotter->outstream);
-	  _emit_float (x0);
-	  _emit_float (y0);
-	  _emit_float (x1);
-	  _emit_float (y1);
-	}
-    }
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_BOX : (int)O_FBOX);
+  _meta_emit_float (x0);
+  _meta_emit_float (y0);
+  _meta_emit_float (x1);
+  _meta_emit_float (y1);
+  _meta_emit_terminator ();
   
   return 0;
 }
