@@ -2,7 +2,6 @@
    libplot.  It draws an object: a circle with center x,y and radius r. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -19,19 +18,11 @@ _m_circle (x, y, r)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d %d\n", 
-		 (int)O_CIRCLE, x, y, r);
-      else
-	{
-	  putc ((int)O_CIRCLE, _plotter->outstream);
-	  _emit_integer (x);
-	  _emit_integer (y);
-	  _emit_integer (r);
-	}
-    }
+  _meta_emit_byte ((int)O_CIRCLE);
+  _meta_emit_integer (x);
+  _meta_emit_integer (y);
+  _meta_emit_integer (r);
+  _meta_emit_terminator ();
   
   return 0;
 }
@@ -50,19 +41,11 @@ _m_fcircle (x, y, r)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %g %g %g\n", 
-		 (int)O_CIRCLE, x, y, r);
-      else
-	{
-	  putc ((int)O_FCIRCLE, _plotter->outstream);
-	  _emit_float (x);
-	  _emit_float (y);
-	  _emit_float (r);
-	}
-    }
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_CIRCLE : (int)O_FCIRCLE);
+  _meta_emit_float (x);
+  _meta_emit_float (y);
+  _meta_emit_float (r);
+  _meta_emit_terminator ();
   
   return 0;
 }
