@@ -16,7 +16,6 @@
      (4) invokes restorestate() to restore an earlier drawing state. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -33,18 +32,10 @@ _m_cont (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d\n", 
-		 (int)O_CONT, x, y);
-      else
-	{
-	  putc ((int)O_CONT, _plotter->outstream);
-	  _emit_integer (x);
-	  _emit_integer (y);
-	}
-    }
+  _meta_emit_byte ((int)O_CONT);
+  _meta_emit_integer (x);
+  _meta_emit_integer (y);
+  _meta_emit_terminator ();
   
   return 0;
 }
@@ -63,18 +54,10 @@ _m_fcont (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %g %g\n", 
-		 (int)O_CONT, x, y);
-      else
-	{
-	  putc ((int)O_FCONT, _plotter->outstream);
-	  _emit_float (x);
-	  _emit_float (y);
-	}
-    }
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_CONT : (int)O_FCONT);
+  _meta_emit_float (x);
+  _meta_emit_float (y);
+  _meta_emit_terminator ();
       
   return 0;
 }
