@@ -3,7 +3,6 @@
    coordinates.  It continues a line. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -20,18 +19,10 @@ _m_contrel (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d\n", 
-		 (int)O_CONTREL, x, y);
-      else
-	{
-	  putc ((int)O_CONTREL, _plotter->outstream);
-	  _emit_integer (x);
-	  _emit_integer (y);
-	}
-    }
+  _meta_emit_byte ((int)O_CONTREL);
+  _meta_emit_integer (x);
+  _meta_emit_integer (y);
+  _meta_emit_terminator ();
   
   return 0;
 }
@@ -50,18 +41,10 @@ _m_fcontrel (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %g %g\n", 
-		 (int)O_CONTREL, x, y);
-      else
-	{
-	  putc ((int)O_FCONTREL, _plotter->outstream);
-	  _emit_float (x);
-	  _emit_float (y);
-	}
-    }
-  
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_CONTREL : (int)O_FCONTREL);
+  _meta_emit_float (x);
+  _meta_emit_float (y);
+  _meta_emit_terminator ();
+      
   return 0;
 }
