@@ -9,7 +9,6 @@
    0xffff, for each of red, green, and blue). */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -26,19 +25,11 @@ _m_fillcolor (red, green, blue)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d %d\n", 
-		 (int)O_FILLCOLOR, red, green, blue);
-      else
-	{
-	  putc ((int)O_FILLCOLOR, _plotter->outstream);
-	  _emit_integer (red);
-	  _emit_integer (green);
-	  _emit_integer (blue);
-	}
-    }
+  _meta_emit_byte ((int)O_FILLCOLOR);
+  _meta_emit_integer (red);
+  _meta_emit_integer (green);
+  _meta_emit_integer (blue);
+  _meta_emit_terminator ();
 
   /* invoke generic method */
   return _g_fillcolor (red, green, blue);
