@@ -4,7 +4,6 @@
    line object to a polyline object. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -21,20 +20,12 @@ _m_line (x0, y0, x1, y1)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf(_plotter->outstream, "%c %d %d %d %d\n", 
-		(int)O_LINE, x0, y0, x1, y1);
-      else
-	{
-	  putc ((int)O_LINE, _plotter->outstream);
-	  _emit_integer (x0);
-	  _emit_integer (y0);
-	  _emit_integer (x1);
-	  _emit_integer (y1);
-	}
-    }
+  _meta_emit_byte ((int)O_LINE);
+  _meta_emit_integer (x0);
+  _meta_emit_integer (y0);
+  _meta_emit_integer (x1);
+  _meta_emit_integer (y1);
+  _meta_emit_terminator ();
   
   return 0;
 }
@@ -53,20 +44,12 @@ _m_fline (x0, y0, x1, y1)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf(_plotter->outstream, "%c %g %g %g %g\n", 
-		(int)O_LINE, x0, y0, x1, y1);
-      else
-	{
-	  putc ((int)O_FLINE, _plotter->outstream);
-	  _emit_float (x0);
-	  _emit_float (y0);
-	  _emit_float (x1);
-	  _emit_float (y1);
-	}
-    }
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_LINE : (int)O_FLINE);
+  _meta_emit_float (x0);
+  _meta_emit_float (y0);
+  _meta_emit_float (x1);
+  _meta_emit_float (y1);
+  _meta_emit_terminator ();
   
   return 0;
 }
