@@ -5,7 +5,6 @@
    current font and fontsize. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -24,20 +23,12 @@ _m_marker (x, y, type, size)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d %d %d\n",
-		 (int)O_MARKER, x, y, type, size);
-      else
-	{
-	  putc ((int)O_MARKER, _plotter->outstream);
-	  _emit_integer (x);
-	  _emit_integer (y);
-	  _emit_integer (type);
-	  _emit_integer (size);
-	}
-    }
+  _meta_emit_byte ((int)O_MARKER);
+  _meta_emit_integer (x);
+  _meta_emit_integer (y);
+  _meta_emit_integer (type);
+  _meta_emit_integer (size);
+  _meta_emit_terminator ();
       
   return 0;
 }
@@ -58,20 +49,12 @@ _m_fmarker (x, y, type, size)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %g %g %d %g\n",
-		 (int)O_MARKER, x, y, type, size);
-      else
-	{
-	  putc ((int)O_FMARKER, _plotter->outstream);
-	  _emit_float (x);
-	  _emit_float (y);
-	  _emit_integer (type);
-	  _emit_float (size);
-	}
-    }
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_MARKER : (int)O_FMARKER);
+  _meta_emit_float (x);
+  _meta_emit_float (y);
+  _meta_emit_integer (type);
+  _meta_emit_float (size);
+  _meta_emit_terminator ();
   
   return 0;
 }
