@@ -5,7 +5,6 @@
    graphics device. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -22,18 +21,10 @@ _m_moverel (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d\n", 
-		 (int)O_MOVEREL, x, y);
-      else
-	{
-	  putc ((int)O_MOVEREL, _plotter->outstream);
-	  _emit_integer (x);
-	  _emit_integer (y);
-	}
-    }
+  _meta_emit_byte ((int)O_MOVEREL);
+  _meta_emit_integer (x);
+  _meta_emit_integer (y);
+  _meta_emit_terminator ();
   
   return 0;
 }
@@ -52,21 +43,10 @@ _m_fmoverel (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	{
-	  putc ((int)O_MOVEREL, _plotter->outstream);	  
-	  fprintf (_plotter->outstream, " %g %g\n", 
-		   x, y);
-	}
-      else
-	{
-	  putc ((int)O_FMOVEREL, _plotter->outstream);
-	  _emit_float (x);
-	  _emit_float (y);
-	}
-    }
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_MOVEREL : (int)O_FMOVEREL);
+  _meta_emit_float (x);
+  _meta_emit_float (y);
+  _meta_emit_terminator ();
   
   return 0;
 }
