@@ -2,7 +2,6 @@
    libplot.  It plots an object: a point with coordinates x,y. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -19,18 +18,10 @@ _m_point (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d\n", 
-		 (int)O_POINT, x, y);
-      else
-	{
-	  putc ((int)O_POINT, _plotter->outstream);
-	  _emit_integer (x);
-	  _emit_integer (y);
-	}
-    }
+  _meta_emit_byte ((int)O_POINT);
+  _meta_emit_integer (x);
+  _meta_emit_integer (y);
+  _meta_emit_terminator ();
       
   return 0;
 }
@@ -49,18 +40,10 @@ _m_fpoint (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %g %g\n", 
-		 (int)O_POINT, x, y);
-      else
-	{
-	  putc ((int)O_FPOINT, _plotter->outstream);
-	  _emit_float (x);
-	  _emit_float (y);
-	}
-    }
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_POINT : (int)O_FPOINT);
+  _meta_emit_float (x);
+  _meta_emit_float (y);
+  _meta_emit_terminator ();
       
   return 0;
 }
