@@ -3,7 +3,6 @@
    coordinates.  It plots an object: a point with specified coordinates. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -20,18 +19,10 @@ _m_pointrel (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %d %d\n", 
-		 (int)O_POINTREL, x, y);
-      else
-	{
-	  putc ((int)O_POINTREL, _plotter->outstream);
-	  _emit_integer (x);
-	  _emit_integer (y);
-	}
-    }
+  _meta_emit_byte ((int)O_POINTREL);
+  _meta_emit_integer (x);
+  _meta_emit_integer (y);
+  _meta_emit_terminator ();
       
   return 0;
 }
@@ -50,18 +41,11 @@ _m_fpointrel (x, y)
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c %g %g\n", 
-		 (int)O_POINTREL, x, y);
-      else
-	{
-	  putc ((int)O_FPOINTREL, _plotter->outstream);
-	  _emit_float (x);
-	  _emit_float (y);
-	}
-    }
+  _meta_emit_byte (_plotter->meta_portable_output ? (int)O_POINTREL : (int)O_FPOINTREL);
+  _meta_emit_float (x);
+  _meta_emit_float (y);
+  _meta_emit_terminator ();
       
   return 0;
 }
+
