@@ -12,7 +12,6 @@
    returned to at that time. */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -28,14 +27,8 @@ _m_savestate ()
       return -1;
     }
 
-  if (_plotter->outstream)
-    {
-      if (_plotter->portable_output)
-	fprintf (_plotter->outstream, "%c\n", 
-		 (int)O_SAVESTATE);
-      else
-	putc ((int)O_SAVESTATE, _plotter->outstream);
-    }
+  _meta_emit_byte ((int)O_SAVESTATE);
+  _meta_emit_terminator ();
   
   /* invoke generic method */
   return _g_savestate ();
