@@ -1,16 +1,12 @@
 /* This file contains the openpl method, which is a standard part of
    libplot.  It opens a Plotter object. */
 
-/* This version is for PS Plotters.  It is just like g_openpl.c, the
-   version used by Plotters that cache each page of graphics rather than
-   emitting graphics in real time.  But it saves a pointer to the first
-   page, since a PS Plotter emits graphics only after all pages of graphics
-   have been drawn, and the Plotter is deleted.  This allows the emitted PS
-   to satisfy the DSC (Document Structuring Conventions).  See
-   p_terminate.c. */
+/* This version is used for Plotters that emit graphics only after all
+   pages of graphics have been drawn, and the Plotter is deleted.  E.g.,
+   for PS Plotters; this delayed output allows the output to satisfy the
+   DSC (Document Structuring Conventions). */
 
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -28,7 +24,7 @@ _p_openpl ()
       return -1;
     }
 
-  /* prepare buffer in which we'll cache Postscript code for this page */
+  /* prepare buffer in which we'll cache code for this page */
   new_page = _new_outbuf ();
 
   if (_plotter->opened == false) /* first page */
@@ -45,13 +41,8 @@ _p_openpl ()
       _plotter->page = new_page;
     }
 
-  /* flag device as open */
-  _plotter->open = true;
-  _plotter->opened = true;
-  (_plotter->page_number)++;
-
-  /* create drawing state, add it as the first member of the linked list */
-  _plotter->savestate();			
+  /* invoke generic method, to e.g. create drawing state */
+  _g_openpl ();
 
   return 0;
 }
