@@ -1,8 +1,12 @@
 /* This file contains the erase method, which is a standard part of
    libplot.  It erases all objects on the graphics device display. */
 
+/* This version is for XDrawablePlotters, and should be identical to
+   y_erase.c (the version for XPlotters) except that XPlotter-specific
+   lines are commented out.  (Search for #if 0...#endif.)  The function
+   y_erase() is of course renamed x_erase() here. */
+
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 /* If we aren't double buffering, this is the number of
@@ -26,9 +30,8 @@ _x_erase ()
   bool head_found;
   int window_width, window_height;
   int i, current_frame;
-  Arg wargs[10];		/* werewolves */
   Colorrecord *cptr, **link = NULL;
-  State *stateptr;
+  pl_DrawState *stateptr;
 
   if (!_plotter->open)
     {
@@ -46,16 +49,17 @@ _x_erase ()
   window_width = (_plotter->imax - _plotter->imin) + 1;
   window_height = (_plotter->jmin - _plotter->jmax) + 1;
 
-  if (_plotter->double_buffering != DBL_NONE)
+  if (_plotter->x_double_buffering != DBL_NONE)
     {
       /* Following two sorts of server-supported double buffering
 	 (DBL_DBE, DBL_MBX) are possible only for X Plotters, not
 	 X Drawable Plotters.  `By hand' double buffering is possible
 	 for both. */
 
+#if 0
 #ifdef HAVE_X11_EXTENSIONS_XDBE_H
 #ifdef HAVE_DBE_SUPPORT
-      if (_plotter->double_buffering == DBL_DBE)
+      if (_plotter->x_double_buffering == DBL_DBE)
 	/* we're using the X double buffering extension */
 	{
 	  XdbeSwapInfo info;
@@ -63,11 +67,11 @@ _x_erase ()
 	  /* Copy current frame of buffered graphics to window.  Implement
 	     this by swapping the front and back buffers for widget's
 	     window.  Former front buffer will become graphics buffer.
-	     Currently, the buffers are `drawable2' (front) and `drawable3'
+	     Currently, the buffers are `x_drawable2' (front) and `x_drawable3'
 	     (back, into which we draw). */
-	  info.swap_window = _plotter->drawable2;
+	  info.swap_window = _plotter->x_drawable2;
 	  info.swap_action = XdbeUndefined;
-	  XdbeSwapBuffers (_plotter->dpy, &info, 1);
+	  XdbeSwapBuffers (_plotter->x_dpy, &info, 1);
 	}
       else
 #endif /* HAVE_DBE_SUPPORT */
@@ -75,7 +79,7 @@ _x_erase ()
 
 #ifdef HAVE_X11_EXTENSIONS_MULTIBUF_H
 #ifdef HAVE_MBX_SUPPORT
-      if (_plotter->double_buffering == DBL_MBX)
+      if (_plotter->x_double_buffering == DBL_MBX)
 	/* we're using the X multibuffering extension */
 	{
 	  Multibuffer multibuf;
@@ -83,33 +87,36 @@ _x_erase ()
 	  /* Copy current frame of buffered graphics to window.  Implement
 	     this by making multibuffer into which we've been drawing the
 	     current multibuffer. */
-	  XmbufDisplayBuffers (_plotter->dpy, 1, &(_plotter->drawable3), 0, 0);
+	  XmbufDisplayBuffers (_plotter->x_dpy, 1, &(_plotter->x_drawable3), 0, 0);
 
 	  /* swap the two multibuffers, making the other one the off-screen
-	     graphics buffer into which we draw (`drawable3') */
-	  multibuf = _plotter->drawable3;
-	  _plotter->drawable3 = _plotter->drawable4;
-	  _plotter->drawable4 = multibuf;
+	     graphics buffer into which we draw (`x_drawable3') */
+	  multibuf = _plotter->x_drawable3;
+	  _plotter->x_drawable3 = _plotter->y_drawable4;
+	  _plotter->y_drawable4 = multibuf;
 	}
       else
 #endif /* HAVE_MBX_SUPPORT */
 #endif /* HAVE_X11_EXTENSIONS_MULTIBUF_H */
+#endif /* 0 */
 
-	/* we must be doing double buffering `by hand', rather than using
-           an X protocol extension */
-      if (_plotter->double_buffering == DBL_BY_HAND)
+      /* we must be doing double buffering `by hand', rather than using
+         an X protocol extension */
+      if (_plotter->x_double_buffering == DBL_BY_HAND)
 	{
 	  /* copy current frame of buffered graphics to drawable(s) */
-	  if (_plotter->drawable1)
-	    XCopyArea (_plotter->dpy, _plotter->drawable3, _plotter->drawable1,
-		       _plotter->drawstate->gc_bg,		   
+	  if (_plotter->x_drawable1)
+	    XCopyArea (_plotter->x_dpy, 
+		       _plotter->x_drawable3, _plotter->x_drawable1,
+		       _plotter->drawstate->x_gc_bg,		   
 		       0, 0,
 		       (unsigned int)window_width, 
 		       (unsigned int)window_height,
 		       0, 0);
-	  if (_plotter->drawable2)
-	    XCopyArea (_plotter->dpy, _plotter->drawable3, _plotter->drawable2,
-		       _plotter->drawstate->gc_bg,		   
+	  if (_plotter->x_drawable2)
+	    XCopyArea (_plotter->x_dpy, 
+		       _plotter->x_drawable3, _plotter->x_drawable2,
+		       _plotter->drawstate->x_gc_bg,		   
 		       0, 0,
 		       (unsigned int)window_width, 
 		       (unsigned int)window_height,
@@ -119,8 +126,8 @@ _x_erase ()
       /* irrespective of which of the three sorts of double buffering is
 	 being performed, clear the (new) graphics buffer, by filling it
 	 with background color */
-      XFillRectangle (_plotter->dpy, _plotter->drawable3, 
-		      _plotter->drawstate->gc_bg,
+      XFillRectangle (_plotter->x_dpy, _plotter->x_drawable3, 
+		      _plotter->drawstate->x_gc_bg,
 		      /* upper left corner */
 		      0, 0,
 		      (unsigned int)window_width, 
@@ -130,35 +137,38 @@ _x_erase ()
     /* not double buffering at all */
     {
       /* erase drawable(s) by filling with background color */
-      if (_plotter->drawable1)
-	XFillRectangle (_plotter->dpy, _plotter->drawable1, 
-			_plotter->drawstate->gc_bg,
+      if (_plotter->x_drawable1)
+	XFillRectangle (_plotter->x_dpy, _plotter->x_drawable1, 
+			_plotter->drawstate->x_gc_bg,
 			/* upper left corner */
 			0, 0,
 			(unsigned int)window_width, (unsigned int)window_height);
-      if (_plotter->drawable2)
-	XFillRectangle (_plotter->dpy, _plotter->drawable2, 
-			_plotter->drawstate->gc_bg,
+      if (_plotter->x_drawable2)
+	XFillRectangle (_plotter->x_dpy, _plotter->x_drawable2, 
+			_plotter->drawstate->x_gc_bg,
 			/* upper left corner */
 			0, 0,
 			(unsigned int)window_width, (unsigned int)window_height);
     }
-  
-  if (_plotter->type == PL_X11)
-    /* If an X Plotter, update background color of canvas widget,
-       irrespective of whether or not we're double buffering.  This fixes
-       things so that if the window is resized to a larger size, the new
-       portions of the window will be filled with the correct color. */
-       {
+
+#if 0
+  /* If an X Plotter, update background color of canvas widget,
+     irrespective of whether or not we're double buffering.  This fixes
+     things so that if the window is resized to a larger size, the new
+     portions of the window will be filled with the correct color. */
+  {
+    Arg wargs[10];		/* werewolves */
+
 #ifdef USE_MOTIF
-	 XtSetArg (wargs[0], XmNbackground, _plotter->drawstate->x_bgcolor);
+    XtSetArg (wargs[0], XmNbackground, _plotter->drawstate->x_bgcolor);
 #else
-	 XtSetArg (wargs[0], XtNbackground, _plotter->drawstate->x_bgcolor);
+    XtSetArg (wargs[0], XtNbackground, _plotter->drawstate->x_bgcolor);
 #endif
-	 XtSetValues (_plotter->toplevel, wargs, (Cardinal)1);
-	 XtSetValues (_plotter->canvas, wargs, (Cardinal)1);
-       }
-  
+    XtSetValues (_plotter->y_toplevel, wargs, (Cardinal)1);
+    XtSetValues (_plotter->canvas, wargs, (Cardinal)1);
+  }
+#endif /* 0 */
+
   /* Flush the color cell cache, to the extent we can.  But heuristically,
      keep in the cache a certain number of cells that aren't strictly
      needed, but which may be needed in the following frames.  There are
@@ -176,6 +186,11 @@ _x_erase ()
 
      In both cases, if a cached cell is to be preserved, it must contain a
      genuine pixel value (the `allocated' flag must be set).
+
+     Side comment: for XDrawable Plotters, unlike XPlotters, it makes no
+     sense to reset the frame number to zero at the conclusion of each
+     openpl..closepl, since we've only got one connection to the X display.
+     So we don't (see x_openpl.c).
    */	   
   cptr = _plotter->x_colorlist;
   _plotter->x_colorlist = NULL;
@@ -189,10 +204,10 @@ _x_erase ()
       cptrnext = cptr->next;
       if (cptr->allocated)
 	{
-	  if (((_plotter->double_buffering == DBL_NONE)
+	  if (((_plotter->x_double_buffering == DBL_NONE)
 	      && i < NUM_KEPT_COLORS)
 	      ||
-	      ((_plotter->double_buffering != DBL_NONE)
+	      ((_plotter->x_double_buffering != DBL_NONE)
 	       && cptr->frame >= current_frame - NUM_KEPT_FRAMES))
 	    {
 	      if (head_found)
@@ -211,7 +226,7 @@ _x_erase ()
 	    /* cached cell contains a genuine pixel value, but it doesn't
 	       meet our criteria, so deallocate it */
 	    {
-	      XFreeColors (_plotter->dpy, _plotter->cmap, 
+	      XFreeColors (_plotter->x_dpy, _plotter->x_cmap, 
 			   &(cptr->rgb.pixel), 1, (unsigned long)0);
 	      free (cptr); 
 	    }
@@ -232,9 +247,12 @@ _x_erase ()
       stateptr->x_bgcolor_status = false;
     }
 
-  /* onward, to the next frame */
+  /* maybe flush X output buffer and handle X events (a no-op for
+     XDrawablePlotters, which is overridden for XPlotters) */
+  _maybe_handle_x_events();
+
+  /* on to next frame */
   _plotter->frame_number++;
 
-  _handle_x_events();
   return 0;
 }
