@@ -2,8 +2,9 @@
    libplot.  It flushes (i.e. pushes onward) all plot commands sent to the
    graphics device. */
 
+/* This version is for XDrawablePlotters (and XPlotters). */
+
 #include "sys-defines.h"
-#include "plot.h"
 #include "extern.h"
 
 int
@@ -19,11 +20,11 @@ _x_flushpl ()
       return -1;
     }
 
-  /* if (_plotter->type == PL_X11) */
-    {
-      XSync (_plotter->dpy, (Bool)false);
-      _handle_x_events();
-    }
+  XSync (_plotter->x_dpy, (Bool)false);
+
+  /* maybe flush X output buffer and handle X events (a no-op for
+     XDrawablePlotters, which is overridden for XPlotters) */
+  _maybe_handle_x_events();
 
   return 0;
 }
